package cn.spatiotemporal.core.async;

import cn.spatiotemporal.core.async.exception.AsyncProcessException;

/**
 * 异步调用统一回调接口
 * @author marquis
 *
 */
@FunctionalInterface
public interface Callback<R> {
	
	/**
	 * 执行成功时触发的回调处理，一般由业务代码自行编写
	 * @param status
	 * @param result
	 */
	public void onSuccess(AsyncResult<R> status, R result);
	
	/**
	 * 执行失败时的回调处理，默认处理为抛出异常
	 * @param status
	 * @param result
	 */
	default void onError(AsyncResult<R> status, R result) {
		throw new AsyncProcessException(status.getMessage());
	}
	
	/**
	 * 默认的回调处理
	 * @param status
	 * @param result
	 */
	default void onDone(AsyncResult<R> status, R result) {
		if (status.isSuccess()) {
			onSuccess(status, result);
		} else {
			onError(status, result);
		}
	}

}
