package cn.spatiotemporal.core.async.agent;

import java.lang.invoke.MethodHandle;
import java.lang.invoke.MethodHandles;
import java.lang.invoke.MethodType;
import java.lang.reflect.Method;

import cn.spatiotemporal.core.async.AsyncResult;
import cn.spatiotemporal.core.async.agent.InvocationHandlerFactory.MethodHandler;
import cn.spatiotemporal.core.async.agent.reflect.Target;

/**
 * 动态代理的各个方法的处理类
 * @author marquis
 *
 */
public class AsyncMethodHandler implements MethodHandler {
	  // handle is effectively final after bindTo has been called.
	  private Target<?> target;
	  
	  MethodHandle handle;

	  public AsyncMethodHandler(Target<?> target, Method method) {
		  this.target = target;
		  MethodHandles.Lookup lookup = MethodHandles.publicLookup();
		  MethodType methodType = MethodType.methodType(method.getReturnType(), method.getParameterTypes());
		  try {
			handle = lookup.findVirtual(method.getDeclaringClass(), method.getName(), methodType);
		} catch (NoSuchMethodException | IllegalAccessException e) {
			throw new IllegalStateException("接口["+method.getDeclaringClass().getSimpleName()+"]里找不到对应的方法["+method.getName()+"]", e);
		}
	  }

	  /**
	   * Bind this handler to a proxy object.  After bound, DefaultMethodHandler#invoke will act as if it was called
	   * on the proxy object.  Must be called once and only once for a given instance of DefaultMethodHandler
	   */
	  public void bindTo(Object proxy) {
	    handle = handle.bindTo(proxy);
	  }

	  /**
	   * Invoke this method.  DefaultMethodHandler#bindTo must be called before the first
	   * time invoke is called.
	   */
	  @Override
	  public Object invoke(Object[] argv) throws Throwable {
		  // 创建callId，异步调用的唯一标识
		  
		  // 序列化参数，构造远程调用的消息头
		  
		  // 通过MQ发送远程调用消息
		  
		  // 构造异步调用状态，需要绑定callId进行存储
		  AsyncResult result = new AsyncResult();
		  
	    return result;
	  }
}
