package cn.spatiotemporal.core.async.agent.reflect;

import cn.spatiotemporal.core.env.Module;

/**
 * 空白的目标模板
 * @author marquis
 *
 * @param <T>
 */
public interface Target<T> {

	/* 
	 * 异步调用接口类型
	 */
	Class<T> type();

	/*
	 * 异步调用的MQ主题
	 */
	String topic();
	
	/*
	 * 调用方的实例
	 */
	long client();

	public static final class EmptyTarget<T> implements Target<T> {

		private final Class<T> type;
		private final String topic;
		private final long client = Module.getInstanceId();

		EmptyTarget(Class<T> type, String topic) {
			this.type = type;
			this.topic = topic;
		}

		public static <T> EmptyTarget<T> create(Class<T> type, String name) {
			return new EmptyTarget<T>(type, name);
		}

		@Override
		public Class<T> type() {
			return type;
		}

		@Override
		public String topic() {
			return topic;
		}
		
		@Override
		public long client() {
			return client;
		}

		@Override
		public boolean equals(Object obj) {
			if (obj instanceof EmptyTarget) {
				EmptyTarget<?> other = (EmptyTarget) obj;
				return type.equals(other.type) && topic.equals(other.topic);
			}
			return false;
		}

		@Override
		public int hashCode() {
			int result = 17;
			result = 31 * result + type.hashCode();
			result = 31 * result + topic.hashCode();
			return result;
		}

		@Override
		public String toString() {
			if (topic.equals("empty:" + type.getSimpleName())) {
				return "EmptyTarget(type=" + type.getSimpleName() + ")";
			}
			return "EmptyTarget(type=" + type.getSimpleName() + ", name=" + topic + ")";
		}
	}
}
