package cn.spatiotemporal.core.async.annotation;

import java.lang.annotation.Documented;
import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

import org.springframework.core.annotation.AliasFor;

/**
 * 异步服务的客户端调用注解
 * @author marquis
 *
 */
@Target(ElementType.TYPE)
@Retention(RetentionPolicy.RUNTIME)
@Documented
public @interface AsyncClient {

	/**
	 * 异步服务的组件名称(MQ的主题名称)
	 */
	@AliasFor("topic")
	String value() default "";

	/**
	 * 异步服务的组件名称(MQ的主题名称)
	 */
	@AliasFor("value")
	String topic() default "";
	
	/**
	 * 通过名称，而非类型来指定实现类
	 */
	@AliasFor("qualifier")
	String name() default "";

	/**
	 * 如果有多个实现类，可以通过<code>@name</code>或<code>@Qualifier</code>的值来区分
	 */
	@AliasFor("name")
	String qualifier() default "";
	
	/**
	 * 重新指定服务端接口或实现类的类型，如果未指定则采用注解所在的接口的类型
	 * @return
	 */
	String type() default "";
}
