package cn.spatiotemporal.core.async.spring;

import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;

import org.springframework.beans.BeansException;
import org.springframework.beans.factory.BeanFactoryUtils;
import org.springframework.beans.factory.DisposableBean;
import org.springframework.beans.factory.FactoryBean;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.boot.autoconfigure.context.PropertyPlaceholderAutoConfiguration;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.context.annotation.AnnotationConfigApplicationContext;
import org.springframework.core.env.MapPropertySource;
import org.springframework.util.Assert;
import org.springframework.util.StringUtils;

import cn.spatiotemporal.core.async.agent.AsyncAgent;
import cn.spatiotemporal.core.async.annotation.AsyncClient;

import ch.qos.logback.core.net.server.Client;
import lombok.Data;
import lombok.EqualsAndHashCode;

/**
 * 异步服务客户端工厂对象
 * 利用动态代理生成异步接口调用的实现类
 * @author marquis
 *
 */
@Data
@EqualsAndHashCode(callSuper = false)
public class AsyncClientFactoryBean implements FactoryBean<Object>, InitializingBean,
DisposableBean, ApplicationContextAware {

	public interface Specification {
		String getName();

		Class<?>[] getConfiguration();
	}

	private Class<?> type;

	private String name;

	private String topic;

	private Map<String, String> attrs;

	private ApplicationContext context;
	
	public AsyncClientFactoryBean(Class<?> type, Map<String, String> attrs) {
		this.type = type;
		this.attrs = attrs;
		this.topic = attrs.get("topic");
		this.name = attrs.get("name");
	}

	@Override
	public void afterPropertiesSet() throws Exception {
		Assert.hasText(this.topic, "Topic must be set");
	}

	@Override
	public void setApplicationContext(ApplicationContext context) throws BeansException {
		this.context = context;
	}

	@Override
	public Object getObject() throws Exception {
		AsyncAgent builder = AsyncAgent.builder(this.topic);

		return builder.build(this.type, this.name, this.attrs);
	}

	@Override
	public Class<?> getObjectType() {
		return this.type;
	}

	@Override
	public boolean isSingleton() {
		return true;
	}

	@Override
	public void destroy() throws Exception {
		// 销毁处理
		
	}
}
