package cn.spatiotemporal.core.config.security;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.UUID;

import org.apache.commons.lang.math.RandomUtils;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

/**
 * 安全配置：模块进程ID
 * @author marquis
 *
 */
@Configuration
public class InstanceConfig {
	
	private static final String instancePath = "com/config";
	
	private static final String instanceFile = "instance.bin";

	/**
	 * 模块运行进程实例ID
	 */
	private final static String compInstanceId;
	
	// 初始化进程实例ID
	static {
		long instanceId = 0;
		try {
			instanceId = loadInstanceId();
		} catch (IOException e) {
			instanceId = UUID.randomUUID().getMostSignificantBits();
			saveInstanceId(instanceId);
		}
		compInstanceId = Long.toHexString(instanceId).toUpperCase();
	}
	
	@Bean
	public String compInstanceId() {
		return compInstanceId;
	}
	
	private static long loadInstanceId() throws IOException {
		File file = new File(instancePath + "/" + instanceFile);
		InputStream is = new FileInputStream(file);
		byte[] bytes = new byte[8];
		is.read(bytes, 0, 8);
		is.close();
		return bytes2long(bytes);
	}
	
	private static void saveInstanceId(long instanceId) {
		try {
			File dir = new File(instancePath);
			if (!dir.exists()) {
				dir.mkdirs();
			}
			File file = new File(instancePath + "/" + instanceFile);
			if (!file.exists()) {
				file.createNewFile();
			}
			OutputStream os = new FileOutputStream(file);
			os.write(long2bytes(instanceId));
			os.close();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * long型转二进制字节流（低位在前，高位在后）
	 * @param value
	 * @return
	 */
	private static byte[] long2bytes(long value) {
		byte[] bytes = new byte[8];
		for (int i = 0; i < 8; i++) {
			bytes[i] = (byte) ((value >> (i * 8)) & 0xFF);
		}
		return bytes;
	}
	
	/**
	 * 二进制字节流转long型
	 * @param bytes
	 * @return
	 */
	private static long bytes2long(byte[] bytes) {
		long value = 0;
		for (int i = 0; i < 8; i++) {
			value += bytes[i] << (i * 8); 
		}
		return value;
	}
}
