package cn.spatiotemporal.core.utils;

import java.util.HashMap;
import java.util.Map;

import org.springframework.cglib.beans.BeanMap;

/**
 * bean与map互转工具类
 * @author marquis
 * @since 2022-02-22
 *
 */
public class BeanMapUtils {
	
	/**
	 * bean转map
	 * @param <T>
	 * @param bean
	 * @return
	 */
	public static <T> Map<String, Object> bean2Map(T bean) {
		if (bean == null) {
			return null;
		}
		Map<String, Object> map = new HashMap<String, Object>();
		BeanMap beanMap = BeanMap.create(bean);
		map.putAll(beanMap);
		return map;
	}
	
	/**
	 * map转bean
	 * @param map
	 * @param clazz
	 * @return
	 */
	public static <T> T map2Bean(Map<String, Object> map, Class<T> clazz) {
		if (map == null || map.isEmpty()) {
			return null;
		}
		try {
			T bean = clazz.newInstance();
			BeanMap.create(bean).putAll(map);
			return bean;
		} catch (InstantiationException e) {
			return null;
		} catch (IllegalAccessException e) {
			return null;
		}
	}

}
