package cn.spatiotemporal.core.utils;

import io.protostuff.LinkedBuffer;
import io.protostuff.ProtostuffIOUtil;
import io.protostuff.Schema;
import io.protostuff.runtime.RuntimeSchema;

import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import org.objenesis.Objenesis;
import org.objenesis.ObjenesisStd;

/**   
 * @ClassName: ProtostuffUtils    
 * @Description: protostuff序列化器工具
 * @date 2020年1月11日 上午10:41:59    
 *     
 * @author  Q.JI
 * @version  
 * @since   JDK 1.8
*/
public class ProtostuffUtils {
	private static ProtostuffUtils instance = null;

	private static Map<Class<?>, Schema<?>> cachedSchema = new ConcurrentHashMap<>();

	private static Objenesis objenesis = new ObjenesisStd(true);

	private ProtostuffUtils() {}

	/**
	 * 
	 * @Title: getInstance    
	 * @Description: TODO(这里用一句话描述这个方法的作用)  
	 * @return  
	 * ProtostuffSerializer
	 */
	public static ProtostuffUtils getInstance() {
		if (instance == null) {
			synchronized (ProtostuffUtils.class) {
				if (instance == null) {
					instance = new ProtostuffUtils();
				}
			}
		}
		return instance;
	}


	/**
	 * 
	 * @Title: serializer    
	 * @Description: TODO(这里用一句话描述这个方法的作用)  
	 * @param obj
	 * @return  
	 * byte[]
	 */
	@SuppressWarnings("unchecked")
	public <T> byte[] serializer(T obj) {
		if(obj == null){
			return new byte[0];
		}
		
		Class<T> cls = (Class<T>) obj.getClass();
		LinkedBuffer buffer = LinkedBuffer.allocate(LinkedBuffer.DEFAULT_BUFFER_SIZE);
		try {
			Schema<T> schema = getSchema(cls);
			return ProtostuffIOUtil.toByteArray(obj, schema, buffer);
		} catch (Exception e) {
			throw new IllegalStateException(e.getMessage(), e);
		} finally {
			buffer.clear();
		}
	}

	/**
	 * 
	 * @Title: deserializer    
	 * @Description: TODO(这里用一句话描述这个方法的作用)  
	 * @param data
	 * @param cls
	 * @return  
	 * T
	 */
	public <T> T deserializer(byte[] data, Class<T> cls) {
		if(isEmpty(data)){
			return null;
		}
		try {
			T message = (T) objenesis.newInstance(cls);
			Schema<T> schema = getSchema(cls);
			ProtostuffIOUtil.mergeFrom(data, message, schema);
			return message;
		} catch (Exception e) {
			throw new IllegalStateException(e.getMessage(), e);
		}
	}

	/**
	 * 
	 * @Title: serializerList    
	 * @Description: TODO(这里用一句话描述这个方法的作用)  
	 * @param list
	 * @return  
	 * byte[]
	 */
	public <T> byte[] serializerList(List<T> list) {
		if (null == list || list.isEmpty()) {
			return new byte[0];
		}
		Pack pack = new Pack();
		pack.setList(list);
		return serializer(pack);
	}

	/**
	 * 
	 * @Title: deserializerList    
	 * @Description: TODO(这里用一句话描述这个方法的作用)  
	 * @param byteArr
	 * @return  
	 * List<?>
	 */
	public <T> List<?> deserializerList(byte[] byteArr) {
		if (isEmpty(byteArr)) {
			return null;
		}
		Pack pack = deserializer(byteArr, Pack.class);
		if(null != pack){
			return pack.getList();
		}else{
			return null;
		}
	}
	
	
	/**
	 * 
	 * @Title: serializerMap    
	 * @Description: TODO(这里用一句话描述这个方法的作用)  
	 * @param map
	 * @return  
	 * byte[]
	 */
	public <T> byte[] serializerMap(Map<?, ?> map) {
		if (null == map || map.size() < 1) {
			return new byte[0];
		}
		Pack pack = new Pack();
		pack.setMap(map);
		return serializer(pack);
	}
	
	
	/**
	 * 
	 * @Title: deserializerMap    
	 * @Description: TODO(这里用一句话描述这个方法的作用)  
	 * @param byteArr
	 * @return  
	 * Map<?,?>
	 */
	public <T> Map<?, ?> deserializerMap(byte[] byteArr) {
		if (isEmpty(byteArr)) {
			return null;
		}
		Pack pack = deserializer(byteArr, Pack.class);
		if(null != pack){
			return pack.getMap();
		}else{
			return null;
		}
	}
	
	@SuppressWarnings("unchecked")
	private <T> Schema<T> getSchema(Class<T> cls) {
		Schema<T> schema = (Schema<T>) cachedSchema.get(cls);
		if (schema == null) {
			schema = RuntimeSchema.createFrom(cls);
			if (schema != null) {
				cachedSchema.put(cls, schema);
			}
		}
		return schema;
	}

	private boolean isEmpty(byte[] data) {
		return (data == null || data.length == 0);
	}
}
